import { createSession, keycloak, setSessionTokenCookie } from '$lib/server/auth';
import { db } from '$lib/server/db';
import { users, type User } from '$lib/server/db/schema/users';
import type { RequestEvent } from '@sveltejs/kit';
import { decodeIdToken, type OAuth2Tokens } from 'arctic';
import { eq } from 'drizzle-orm';
import { v7 as uuid } from 'uuid';

export async function GET(event: RequestEvent): Promise<Response> {
	const code = event.url.searchParams.get('code');
	const state = event.url.searchParams.get('state');
	const storedState = event.cookies.get('keycloak_oauth_state') ?? null;
	const codeVerifier = event.cookies.get('keycloak_code_verifier') ?? null;
	if (code === null || state === null || storedState === null || codeVerifier === null) {
		return new Response(null, {
			status: 400
		});
	}
	if (state !== storedState) {
		return new Response(null, {
			status: 400
		});
	}

	let tokens: OAuth2Tokens;
	try {
		tokens = await keycloak.validateAuthorizationCode(code, codeVerifier);
	} catch {
		// Invalid code or client credentials
		return new Response(null, {
			status: 400
		});
	}
	const claims = decodeIdToken(tokens.idToken());
	if (
		!(
			'sub' in claims &&
			typeof claims.sub === 'string' &&
			'name' in claims &&
			typeof claims.name === 'string' &&
			'email' in claims &&
			typeof claims.email === 'string'
		)
	) {
		// Invalid token
		return new Response(null, {
			status: 400
		});
	}
	const keycloakUserId = claims.sub;
	const username = claims.name;
	const email = claims.email;

	// TODO: Replace this with your own DB query.
	const existingUser = await getUserFromKeycloakId(keycloakUserId);

	if (existingUser !== null) {
		const session = await createSession(existingUser.id);
		setSessionTokenCookie(event, session.token, session.session.expires_at);
		return new Response(null, {
			status: 302,
			headers: {
				Location: '/'
			}
		});
	}

	const user = await createUser(keycloakUserId, username, email);

	const session = await createSession(user.id);
	setSessionTokenCookie(event, session.token, session.session.expires_at);
	return new Response(null, {
		status: 302,
		headers: {
			Location: '/'
		}
	});
}

async function getUserFromKeycloakId(keycloakUserId: string): Promise<User | null> {
	const [result] = await db
		.select({
			user: users
		})
		.from(users)
		.where(eq(users.oauth_id, keycloakUserId));
	return result ? result.user : null;
}

async function createUser(keycloakUserId: string, username: string, email: string): Promise<User> {
	const [result] = await db
		.insert(users)
		.values({
			id: uuid(),
			oauth_id: keycloakUserId,
			username,
			email,
			created_at: new Date(Date.now()),
			deleted_at: new Date(),
			updated_at: new Date(Date.now())
		})
		.returning();

	// add user to default role

	console.log(JSON.stringify(result));
	return result;
}
