import { db } from '$lib/server/db';
import type { Actions, PageServerLoad } from './$types';
import { competitions, getCompetitionWithAll } from '$lib/server/db/schema/competitions';
import { error, redirect } from '@sveltejs/kit';
import { eq } from 'drizzle-orm';
import { and } from 'drizzle-orm';
import { teams } from '$lib/server/db/schema/teams';
import { superValidate } from 'sveltekit-superforms';
import { zod } from 'sveltekit-superforms/adapters';
import { formSchema } from '$lib/components/rounds/create-schema';

export const load: PageServerLoad = async ({ params, locals }) => {
	try {
		const id = Number.parseInt(params.id);

		const competition = await getCompetitionWithAll(id);

		if (!competition) {
			throw error(404, 'Competition not found');
		}

		if (competition.owner !== locals.user.id) {
			throw error(403, 'You are not authorized to view this competition');
		}

		return {
			competition: competition,
			createForm: await superValidate(zod(formSchema))
		};
	} catch (e) {
		return redirect(302, '/');
	}
};

export const actions = {
	addTeam: async (event) => {
		const data = await event.request.formData();

		const teamName = data.get('team_name')?.toString();

		if (!teamName) {
			throw error(400, 'Team name is required');
		}

		// get competition id from url
		const id = Number.parseInt(event.params.id, 10);

		if (id === -1) {
			throw error(400, 'Invalid competition id');
		}

		let [competition] = await db
			.select()
			.from(competitions)
			.leftJoin(teams, eq(competitions.id, teams.competition_id))
			.where(and(eq(competitions.id, id), eq(competitions.owner, event.locals.user.id)));

		if (!competition) {
			throw error(404, 'Competition not found');
		}

		// add team to competition's teams array
		await db.insert(teams).values({
			competition_id: id,
			name: teamName
		});
		const comp = await getCompetitionWithAll(id);

		if (!comp || comp.owner !== event.locals.user.id) {
			throw error(403, 'You are not authorized to update this competition');
		}

		return {
			comp
		};
	},
	deleteTeam: async (event) => {
		const data = await event.request.formData();

		const team = data.get('id')?.toString();

		if (!team) {
			throw error(400, 'Team name is required');
		}

		const teamId = Number.parseInt(team, 10);

		// get competition id from url
		const id = Number.parseInt(event.params.id, 10);

		if (id === -1) {
			throw error(400, 'Invalid competition id');
		}

		let [competition] = await db
			.select()
			.from(competitions)
			.leftJoin(teams, eq(competitions.id, teams.competition_id))
			.where(and(eq(competitions.id, id), eq(competitions.owner, event.locals.user.id)));

		if (!competition) {
			throw error(404, 'Competition not found');
		}

		// remove team to competition's teams array
		await db.delete(teams).where(and(eq(teams.id, teamId), eq(teams.competition_id, id)));
		const comp = await getCompetitionWithAll(id);

		if (!comp || comp.owner !== event.locals.user.id) {
			throw error(403, 'You are not authorized to update this competition');
		}

		return {
			comp
		};
	},
	updateDescription: async (event) => {
		const data = await event.request.formData();
		const description = data.get('description')?.toString();

		// get competition id from url
		const id = Number.parseInt(event.params.id, 10);

		if (id === -1) {
			console.log(id);
			throw error(400, 'Invalid competition id');
		}

		let [competition] = await db
			.select()
			.from(competitions)
			.leftJoin(teams, eq(competitions.id, teams.competition_id))
			.where(and(eq(competitions.id, id), eq(competitions.owner, event.locals.user.id)));

		if (!competition) {
			console.log(competition);
			throw error(404, 'Competition not found');
		}

		// update competition's description
		await db.update(competitions).set({ description }).where(eq(competitions.id, id));

		const comp = await getCompetitionWithAll(id);

		if (!comp || comp.owner !== event.locals.user.id) {
			throw error(403, 'You are not authorized to update this competition');
		}

		return {
			comp
		};
	}
} satisfies Actions;
